/**
 * 
 */
package org.springblade.cube.controller;

import java.util.ArrayList;
import java.util.Base64;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.cube.dto.CubeDataParam;
import org.springblade.cube.dto.CubeDataPermRelation;
import org.springblade.cube.dto.CubeDataWithPerm;
import org.springblade.cube.dto.CubeTableName;
import org.springblade.cube.dto.OrderByParam;
import org.springblade.cube.dto.PreviewResult;
import org.springblade.cube.dto.TableLinkJoin;
import org.springblade.cube.dto.TableLinkJoin.JoinKey;
import org.springblade.cube.dto.VirtualColumnParam;
import org.springblade.cube.enums.ColumnAttrEnum;
import org.springblade.cube.enums.DateFormatEnum;
import org.springblade.cube.enums.FunctionTypeEnum;
import org.springblade.cube.enums.TableTypeEnum;
import org.springblade.cube.enums.ViewColumnTypeEnum;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeDatasouceFunction;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.cube.service.CubeDataPermissionService;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.CubeDatasouceFunctionService;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.enums.DsTypeEnum;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.dto.CubePermParseDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Tuple;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@RestController
@RequestMapping("/cube/data")
@Api(value = "数据集接口", tags = "数据集相关接口")
public class CubeDataController extends BladeController {

	@Autowired
	private IReportDatasourceService reportDatasourceService;

	@Autowired
	private DynamicTableService dynamicTableService;

	@Autowired
	private CubeDataService cubeDataService;

	@Autowired
	private DataPermClient dataPermClient;

	@Autowired
	private CubeDataPermissionService cubeDataPermissionService;

	@Autowired
	private CubeDatasouceFunctionService cubeDatasouceFunctionService;

	/**
	 * 
	 * @param cubeData
	 * @param query
	 * @return
	 */
	@GetMapping("/list")
	@ApiOperation(value = "数据集列表")
	public R<IPage<CubeDataWithPerm>> list(CubeData cubeData, Query query) {
		// 判断是否超级权限组「拥有所有权限」
		long userId = getUser().getUserId();
		Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(userId);
		LambdaQueryWrapper<CubeData> queryWrapper = Condition.getQueryWrapper(cubeData).lambda().orderByDesc(CubeData::getId);
		// 非超级权限组用户，查询有权限的数据集列表
		if (checkSuperPermRet == null || !checkSuperPermRet) {
			queryWrapper.and(wrapper -> {
				wrapper.eq(CubeData::getCreateUserId, userId).or().eq(CubeData::getPublicStatus, 1)
				.or().inSql(CubeData::getId, "select a.cube_data_id from cube_data_permission a inner join data_permission_user b on a.data_perm_id = b.data_perm_id where b.user_id = " + userId);
			});
		}
		//查询数据
		IPage<CubeData> pageResult = cubeDataService.page(Condition.getPage(query), queryWrapper);
		IPage<CubeDataWithPerm> resultPage = null;
		//不为空，设置权限组名称
		if(pageResult != null && CollectionUtil.isNotEmpty(pageResult.getRecords())) {
			resultPage = new Page<>(pageResult.getCurrent(), pageResult.getSize(), pageResult.getTotal());
			List<Integer> cubeIdList = pageResult.getRecords().stream().map(CubeData::getId).collect(Collectors.toList());
			Map<Integer, List<CubeDataPermRelation>> permRelaMap = Optional.ofNullable(cubeDataPermissionService.selectListByCubeIdList(cubeIdList)).orElse(new ArrayList<>()).stream().collect(Collectors.groupingBy(CubeDataPermRelation::getCubeDataId));
			List<CubeDataWithPerm> records = new ArrayList<>();
			pageResult.getRecords().forEach(item -> {
				CubeDataWithPerm cubeDataWithPerm = BeanUtil.copyProperties(item, CubeDataWithPerm.class);
				cubeDataWithPerm.setDataPermRelationList(permRelaMap.get(item.getId()));
				records.add(cubeDataWithPerm);
			});
			resultPage.setRecords(records);
		}
		return R.data(resultPage);
	}

	/**
	 * 
	 * @param cubeDataId
	 * @return
	 */
	@GetMapping("/info")
	@ApiOperation(value = "根据ID查询数据集详情")
	public R<CubeData> info(@RequestParam Integer cubeDataId) {
		return R.data(cubeDataService.getById(cubeDataId));
	}

	/**
	 * 数据预览
	 * 
	 * @param queryParam
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	@ApiOperation(value = "预览数据")
	@PostMapping("/preview")
	public R<PreviewResult> previewData(@RequestBody String value) throws Exception {
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		CubeDataParam queryParam = JSON.parseObject(decoderStr, CubeDataParam.class);
		Integer datasourceId = queryParam.getDatasourceId();
		if (datasourceId == null || datasourceId <= 0) {
			return R.fail("数据源ID不能为空");
		}
		// 前提:查询数据源
		ReportDatasource reportDatasource = reportDatasourceService.getById(datasourceId);
		if (reportDatasource == null) {
			return R.fail("数据源不存在");
		}
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(reportDatasource.getDsType());
		
		// 查询自定义函数列表[用于对虚拟字段中带有自定义函数的解析处理]
		List<CubeDatasouceFunction> functionList = cubeDatasouceFunctionService
				.list(Wrappers.<CubeDatasouceFunction>lambdaQuery().eq(CubeDatasouceFunction::getDsType, dsTypeEnum)
						.eq(CubeDatasouceFunction::getFuncType, FunctionTypeEnum.custom));
		
		// 获取字段列表
		List<CubeTableColumn> columnList = queryParam.getColumnList();
		Map<String, CubeTableColumn> columnMap = columnList.stream()
				.collect(Collectors.toMap(CubeTableColumn::getColumnTitle, t -> t, (n, n1) -> n));
		
		// 1.获取数据集,开始拼接FROM表相关SQL
		String fromSql = _buildFromSql(queryParam.getCubeData(), columnMap, functionList);
//		log.info("fromSql:" + fromSqlBuffer.toString());

		StringBuilder whereSqlBuffer = new StringBuilder("1=1");
		// 组装权限组过滤条件
		if (queryParam.getCubeDataId() != null) {
			CubePermParseDto cubePermParseDto = cubeDataPermissionService
					.parseCubePermission(queryParam.getCubeDataId(), getUser());
			if (StrUtil.isNotBlank(cubePermParseDto.getFilterSql())) {
				whereSqlBuffer.append(" AND " + cubePermParseDto.getFilterSql());
			}
		}

		Map<String, String> fieldDict = new HashMap<String, String>();
		Integer[] indexArr = { 0 };
		// 初始化别名
		columnList.forEach(f -> {
			String columnAliasName = CubeConstants.COLUMN_ALIAS_NAME_PREFIX + (indexArr[0]++);
			f.setColumnAliasName(columnAliasName);
			fieldDict.put(columnAliasName, f.getColumnTitle());
		});

		List<String> selectSqlList = new ArrayList<String>();
		for (CubeTableColumn tableColumn : columnList) {
			if (tableColumn.getHiddenStatus() == 1) {
				continue;
			}
			String columnSql = tableColumn.getTableAliasName() + "." + tableColumn.getColumnName();
			// 虚拟字段
			if (tableColumn.getColumnAttr() == ColumnAttrEnum.VIRTUAL) {
				// 计算字段中：自定义函数前置解析处理
				Tuple customFunctionSqlTuple = _parseCustomFunctionSql(queryParam.getCubeData(),
						tableColumn.getColumnTitle(), tableColumn.getColumnFormula(), functionList);
				String finColumnFormula = customFunctionSqlTuple != null ? customFunctionSqlTuple.get(0)
						: tableColumn.getColumnFormula();
				// 如果包含聚合函数，不执行
				if (StrUtil.containsAny(finColumnFormula.toLowerCase(), CubeConstants.AGGREGATION_FUNCTIONS)) {
					columnSql = CubeConstants.DEFAULT_SHOW_CHAR;
				} else {
					VirtualColumnParam virtualColumnParam = CubeUtils.formatColumnFormula(finColumnFormula, columnMap);
					columnSql = virtualColumnParam.getColumnSql();
				}
			}
			// 数据类型转换
			columnSql = CubeUtils.convertDataType(dsTypeEnum, tableColumn.getOldViewDataType(),
					tableColumn.getViewDataType(), columnSql, tableColumn.getTimeDataFormat());
			// 格式化日期，默认日期格式是：yyyy_MM_ddhh$mm$ss
			if (tableColumn.getTimeDataFormat() != null
					&& tableColumn.getTimeDataFormat() != DateFormatEnum.yyyy_MM_ddhh$mm$ss
					&& tableColumn.getViewDataType() == ViewColumnTypeEnum.DATETIME
					&& !CubeConstants.DEFAULT_SHOW_CHAR.equals(columnSql)) {
				columnSql = CubeUtils.formatTimeColumn(tableColumn.getTimeDataFormat(), dsTypeEnum, columnSql);
			}
			// 格式化数字
			if (tableColumn.getNumberDataFormat() != null && tableColumn.getViewDataType() == ViewColumnTypeEnum.NUMBER
					&& !CubeConstants.DEFAULT_SHOW_CHAR.equals(columnSql)) {
				columnSql = String.format(tableColumn.getNumberDataFormat().getMethod(), columnSql);
				// 拼接百分号
				columnSql = CubeUtils.percentConcat(tableColumn.getNumberDataFormat(), dsTypeEnum, columnSql);
			}
			selectSqlList.add(columnSql + " as \"" + tableColumn.getColumnAliasName() + "\"");
		}
		String selectSql = String.join(",", selectSqlList);
		// 3.获取OrderBy信息,开始做排序相关SQL
		List<OrderByParam> orderByList = queryParam.getOrderByList();
		String orderBySql = "";
		if (orderByList != null && !orderByList.isEmpty()) {
			List<String> orderBySqlList = new ArrayList<String>();
			for (OrderByParam orderByParam : orderByList) {
				CubeTableColumn cubeTableColumn = columnMap.get(orderByParam.getColumnTitle());
				orderBySqlList.add(cubeTableColumn.getColumnAliasName() + " " + orderByParam.getOrderBy().name());
			}
			if (!orderBySqlList.isEmpty()) {
				orderBySql = "order by " + String.join(",", orderBySqlList);
			}
		}
		String querySql = String.format(CubeConstants.SELECT_SQL_FORMAT, selectSql, fromSql, whereSqlBuffer.toString(),
				orderBySql);
//		log.info("QuerySQL:" + querySql);
		if (CubeUtils.checkSqlInject(querySql)) {
			return R.fail("有SQL注入风险，请检查");
		}
		List<Map<String, Object>> resultList = null;
		if (queryParam.getLimit() > 0) { // 分页查询
			resultList = dynamicTableService.selectPageList(new Page(0, queryParam.getLimit()).setSearchCount(false),
					reportDatasource, querySql);
		} else {
			resultList = dynamicTableService.selectList(reportDatasource, querySql);
		}
		return R.data(new PreviewResult(resultList, fieldDict, queryParam.getCubeData().getCubeName()), querySql);
	}

	/**
	 * 保存或修改数据集
	 * 
	 * @param queryParam
	 * @return
	 */
	@ApiOperation(value = "保存或修改数据集")
	@PostMapping("/saveOrUpdate")
	public R<Integer> saveOrUpdate(@RequestBody String value) throws Exception {
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		CubeDataParam cubeDataParam = JSON.parseObject(decoderStr, CubeDataParam.class);
		Integer datasourceId = cubeDataParam.getDatasourceId();
		if (datasourceId == null || datasourceId <= 0) {
			return R.fail("数据源ID不能为空");
		}
		// 前提:查询数据源
		ReportDatasource reportDatasource = reportDatasourceService.getById(datasourceId);
		if (reportDatasource == null) {
			return R.fail("数据源不存在");
		}
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(reportDatasource.getDsType());

		// 获取字段列表,开始拼接SELECT字段相关SQL
		List<CubeTableColumn> columnList = cubeDataParam.getColumnList();
		Map<String, CubeTableColumn> columnMap = columnList.stream()
				.collect(Collectors.toMap(CubeTableColumn::getColumnTitle, t -> t, (n, n1) -> n));
		// 查询自定义函数列表[用于对虚拟字段中带有自定义函数的解析处理]
		List<CubeDatasouceFunction> functionList = cubeDatasouceFunctionService
				.list(Wrappers.<CubeDatasouceFunction>lambdaQuery().eq(CubeDatasouceFunction::getDsType, dsTypeEnum)
						.eq(CubeDatasouceFunction::getFuncType, FunctionTypeEnum.custom));

		// 1.获取数据集,开始拼接FROM表相关SQL
		CubeData cubeData = cubeDataParam.getCubeData();
		if(cubeData.getId() == null) {
			cubeData.setCreateBy(SecureUtil.getNickName());
			cubeData.setCreateTime(new Date());
			cubeData.setCreateUserId(SecureUtil.getUserId());
		}
		String fromSql = _buildFromSql(cubeData, columnMap, functionList);
		cubeData.setQuerySql(fromSql);
		log.info("fromSql:" + fromSql);
		// @TODO 组装全局过滤条件
//		StringBuilder whereSqlBuffer = new StringBuilder("1=1");
//		log.info("whereSql:" + whereSqlBuffer.toString());

		// 处理虚拟字段保存字段名和sql
		for (CubeTableColumn tableColumn : columnList) {
			String columnSql = tableColumn.getTableAliasName() + "." + tableColumn.getColumnName();
			if (tableColumn.getColumnAttr() == ColumnAttrEnum.VIRTUAL) {
				// 计算字段中：自定义函数前置解析处理
				Tuple customFunctionSqlTuple = _parseCustomFunctionSql(cubeData, tableColumn.getColumnTitle(),
						tableColumn.getColumnFormula(), functionList);
				String finColumnFormula = customFunctionSqlTuple != null ? customFunctionSqlTuple.get(0)
						: tableColumn.getColumnFormula();
				// 设置计算字段，是否包含聚合函数
				tableColumn.setVirtualAggregationStatus(
						StrUtil.containsAny(finColumnFormula.toLowerCase(), CubeConstants.AGGREGATION_FUNCTIONS) ? 1
								: 0);
				tableColumn.setVirtualOnlyZhibiaoStatus(
						customFunctionSqlTuple != null ? customFunctionSqlTuple.get(1) : 0);
				// 格式化虚拟字段
				VirtualColumnParam virtualColumnParam = CubeUtils.formatColumnFormula(finColumnFormula, columnMap);
				tableColumn.setColumnName(String.join(",", virtualColumnParam.getColumnNames()));
				columnSql = virtualColumnParam.getColumnSql();
			}
			// 格式转换
			tableColumn.setColumnSql(CubeUtils.convertDataType(dsTypeEnum, tableColumn.getOldViewDataType(),
					tableColumn.getViewDataType(), columnSql, tableColumn.getTimeDataFormat()));
			if (CubeUtils.checkSqlInject(tableColumn.getColumnSql())) {
				return R.fail("[" + tableColumn.getColumnComment() + "]有SQL注入风险，请检查");
			}
		}
		return cubeDataService.doSaveOrUpdate(cubeData, columnList, cubeDataParam.getDataPermIdList(), getUser());
	}

	/**
	 * 构造fromSql
	 * 
	 * @param cubeData
	 * @param columnMap
	 * @return
	 */
	private String _buildFromSql(CubeData cubeData, Map<String, CubeTableColumn> columnMap, List<CubeDatasouceFunction> functionList) {
		StringBuilder fromSqlBuffer = new StringBuilder();
		if (cubeData.getTableType() == TableTypeEnum.LOGIC) { // 有关联表
			List<TableLinkJoin> tableLinkJoinList = JSON.parseArray(cubeData.getLinksJoin(), TableLinkJoin.class);
			if (CollectionUtil.isEmpty(tableLinkJoinList)) {
				throw new ServiceException("你还没有指定关联表");
			}
			for (int i = 0; i < tableLinkJoinList.size(); i++) {
				TableLinkJoin tableLinkJoin = tableLinkJoinList.get(i);
				if (i == 0) {
					fromSqlBuffer.append(tableLinkJoin.getSource()).append(" ").append(tableLinkJoin.getSourceId());
				}
				fromSqlBuffer.append(tableLinkJoin.getJoinType().getSql()).append(tableLinkJoin.getTarget()).append(" ")
						.append(tableLinkJoin.getTargetId());
				// 拼接ON
				fromSqlBuffer.append(" on ");
				int j = 1, size = tableLinkJoin.getJoinKeys().size();
				for (JoinKey joinKey : tableLinkJoin.getJoinKeys()) {
					if (joinKey.isSourceKeyVtState()) {
						// 计算字段中：自定义函数前置解析处理
						Tuple customFunctionSqlTuple = _parseCustomFunctionSql(cubeData, "", joinKey.getSourceKey(), functionList);
						String finColumnFormula = customFunctionSqlTuple != null ? customFunctionSqlTuple.get(0) : joinKey.getSourceKey();
						fromSqlBuffer.append(CubeUtils.formatColumnFormula(finColumnFormula, columnMap).getColumnSql());
					} else {
						fromSqlBuffer.append(tableLinkJoin.getSourceId()).append(".").append(joinKey.getSourceKey());
					}
					fromSqlBuffer.append("=");
					if (joinKey.isTargetKeyVtState()) {
						// 计算字段中：自定义函数前置解析处理
						Tuple customFunctionSqlTuple = _parseCustomFunctionSql(cubeData, "", joinKey.getTargetKey(), functionList);
						String finColumnFormula = customFunctionSqlTuple != null ? customFunctionSqlTuple.get(0) : joinKey.getTargetKey();
						fromSqlBuffer.append(CubeUtils.formatColumnFormula(finColumnFormula, columnMap).getColumnSql());
//						fromSqlBuffer.append(
//								CubeUtils.formatColumnFormula(joinKey.getTargetKey(), columnMap).getColumnSql());
					} else {
						fromSqlBuffer.append(tableLinkJoin.getTargetId()).append(".").append(joinKey.getTargetKey());
					}
					if (j++ != size) {
						fromSqlBuffer.append(" and ");
					}
				}
			}
		} else {
			List<CubeTableName> tableLinkJoinList = JSON.parseArray(cubeData.getCubeTableNames(), CubeTableName.class);
			if (CollectionUtil.isEmpty(tableLinkJoinList)) {
				throw new ServiceException("你还没有指定关联表");
			}
			CubeTableName tableName = tableLinkJoinList.get(0);
			fromSqlBuffer.append(tableName.getTableName()).append(" ").append(tableName.getTableAliasName());
		}
		return fromSqlBuffer.toString();
	}

	/**
	 * 解析虚拟字段公式
	 * 
	 * @param columnTitle   - 错误提示使用
	 * @param columnFormula - 虚拟字段公式
	 * @return
	 */
	private Tuple _parseCustomFunctionSql(CubeData cubeData, String columnTitle, String columnFormula,
			List<CubeDatasouceFunction> functionList) {
		if (CollectionUtil.isEmpty(functionList))
			return null;
		// 获取主表名和别名
		String masterTableName = "", masterAliasName = "";
		if (cubeData.getTableType() == TableTypeEnum.LOGIC) { // 有关联表
			List<TableLinkJoin> tableLinkJoinList = JSON.parseArray(cubeData.getLinksJoin(), TableLinkJoin.class);
			if (CollectionUtil.isEmpty(tableLinkJoinList)) {
				throw new ServiceException("你还没有指定关联表");
			}
			TableLinkJoin tableLinkJoin = tableLinkJoinList.get(0);
			masterTableName = tableLinkJoin.getSourceId();
			masterAliasName = tableLinkJoin.getSource();
		} else {
			List<CubeTableName> tableLinkJoinList = JSON.parseArray(cubeData.getCubeTableNames(), CubeTableName.class);
			if (CollectionUtil.isEmpty(tableLinkJoinList)) {
				throw new ServiceException("你还没有指定关联表");
			}
			CubeTableName tableName = tableLinkJoinList.get(0);
			masterTableName = tableName.getTableName();
			masterAliasName = tableName.getTableAliasName();
		}
		CubeDatasouceFunction function = functionList.stream().filter(item -> columnFormula.contains(item.getMethodName()+"(")).findAny().orElse(null);
		if (function != null) {
			// 获取自定义函数的参数
			String subBetweenStr = StrUtil
					.removeSuffix(StrUtil.removePrefixIgnoreCase(columnFormula, function.getMethodName() + "("), ")");
			String[] params = subBetweenStr.split(",");
			Map<String, String> paramMap = IntStream.range(0, params.length).boxed()
					.collect(Collectors.toMap(t -> t.toString(), i -> params[i]));
			paramMap.put("master_table_name", masterTableName); // 主表名
			paramMap.put("master_alias_name", masterAliasName); // 主表别名
			String customSql = StrUtil.format(function.getFuncSql(), paramMap);
			if (customSql.contains("{") || customSql.contains("}")) {
				throw new ServiceException("虚拟字段[" + columnTitle + "]参数不对应");
			}
			return new Tuple(customSql, function.getSubQueryStatus());
		}
		return null;
	}

	/**
	 * 列权限校验
	 * 
	 * @param fieldDict
	 * @param resultList
	 * @param columnList
	 * @param excludeColumnIdList
	 */
	void _formatExcludeColumn(List<JSONObject> resultList, List<CubeTableColumn> columnList,
			Set<Integer> excludeColumnIdList) {
		// 列权限，删除掉不显示的列
		if (CollectionUtil.isNotEmpty(excludeColumnIdList) && CollectionUtil.isNotEmpty(resultList)) {
			Map<Integer, String> fieldIdMap = columnList.stream()
					.collect(Collectors.toMap(CubeTableColumn::getId, CubeTableColumn::getColumnAliasName));
			List<String> excludeColumnAliasList = excludeColumnIdList.stream().map(it -> fieldIdMap.get(it))
					.filter(Objects::nonNull).collect(Collectors.toList());
			if (CollectionUtil.isNotEmpty(excludeColumnAliasList)) {
				// 格式化数据
				for (JSONObject jsonObj : resultList) {
					excludeColumnAliasList.stream().forEach(item -> jsonObj.remove(item));
				}
			}
		}
	}
}
