/**
 * 
 */
package org.springblade.cube.enums;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.springblade.cube.model.FileStructure;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.report.enums.DsTypeEnum;

import cn.hutool.core.util.StrUtil;

/**
 * @author wookvn
 *
 */
public enum DbCreateTableEnum {

	Mysql("create table %s (%s) comment '%s'", "", ""){
		@Override
		public List<String> createTableSql(String tableName, String tableComment,
				List<FileStructure> structureList) {
			StringBuilder fieldSql = new StringBuilder();
			structureList.stream().forEach(f ->{
				fieldSql.append(String.format("%s %s %s comment '%s',", 
						f.getColumnName(), DbMysqlEnums.ColumnType.valueOf(f.getColumnType().name()).columnType, 
						f.isPrimaryKey() ? "primary key" : "", f.getComment()));
			});
			fieldSql.deleteCharAt(fieldSql.length() - 1);
			return Arrays.asList(String.format(createTableSql, tableName, fieldSql.toString(), tableComment));
		}
	},
	
	//create table user_test(ID UInt64, NAME Nullable(String), PRIMARY KEY (ID)) ENGINE=MergeTree COMMENT '测试用户表';
	Clickhouse("create table %s (%s) ENGINE=MergeTree COMMENT '%s'", "", ""){
		@Override
		public List<String> createTableSql(String tableName, String tableComment,
				List<FileStructure> structureList) {
			StringBuilder fieldSql = new StringBuilder();
			String primaryKey = "";
			for(FileStructure item : structureList) {
				String _columnType = DbClickHouseEnums.ColumnType.valueOf(item.getColumnType().name()).columnType;
				String columnType = StrUtil.equalsAnyIgnoreCase(item.getColumnName(), CubeConstants.COLUMN_USER_ID, CubeConstants.COLUMN_PRIMARY_KEY)
						? _columnType : "Nullable(" + _columnType + ")";
				fieldSql.append(String.format("%s %s COMMENT '%s',", item.getColumnName(), columnType, item.getComment()));
				if(item.isPrimaryKey()) {
					primaryKey = item.getColumnName();
				}
			}
			fieldSql.append("PRIMARY KEY(" + primaryKey + ")");
			return Arrays.asList(String.format(createTableSql, tableName, fieldSql.toString(), tableComment));
		}
	},
	
	Oracle("create table %s (%s)", 
			"COMMENT ON table %s IS '%s'", 
			"COMMENT ON column %s.%s IS '%s'"){
		@Override
		public List<String> createTableSql(String tableName, String tableComment,
				List<FileStructure> structureList) {
			List<String> sqlList = new LinkedList<String>();
			StringBuilder fieldSql = new StringBuilder();
			structureList.stream().forEach(f ->{
				fieldSql.append(String.format("%s %s %s,", 
						f.getColumnName(), DbOracleEnums.ColumnType.valueOf(f.getColumnType().name()).columnType,
						f.isPrimaryKey() ? "primary key" : ""));
				//添加字段注释SQL
				sqlList.add(String.format(createColumnCommentSql, tableName, f.getColumnName(), f.getComment()));
			});
			fieldSql.deleteCharAt(fieldSql.length() - 1);
			//创建表SQL
			sqlList.add(0, String.format(createTableSql, tableName, fieldSql.toString()));
			//添加表注释SQL
			sqlList.add(1, String.format(createTableCommentSql, tableName, tableComment));
			return sqlList;
		}
	},
	
	SqlServer("create table %s (%s)", 
			"EXEC sp_addextendedproperty N'MS_Description', N'%s', N'user', N'dbo', N'table', N'%s', NULL, NULL", 
			"EXEC sp_addextendedproperty N'MS_Description', N'%s', N'user', N'dbo', N'table', N'%s', N'COLUMN', N'%s'"){
		@Override
		public List<String> createTableSql(String tableName, String tableComment,
				List<FileStructure> structureList) {
			List<String> sqlList = new LinkedList<String>();
			StringBuilder fieldSql = new StringBuilder();
			structureList.stream().forEach(f ->{
				fieldSql.append(String.format("%s %s %s,", 
						f.getColumnName(), DbSqlServerEnums.ColumnType.valueOf(f.getColumnType().name()).columnType,
						f.isPrimaryKey() ? "primary key" : ""));
				//添加字段注释SQL
				sqlList.add(String.format(createColumnCommentSql, f.getComment(), tableName, f.getColumnName()));
			});
			fieldSql.deleteCharAt(fieldSql.length() - 1);
			//创建表SQL
			sqlList.add(0, String.format(createTableSql, tableName, fieldSql.toString()));
			//添加表注释SQL
			sqlList.add(1, String.format(createTableCommentSql, tableComment, tableName));
			return sqlList;
		}
	};
	
	String createTableSql;
	String createTableCommentSql;
	String createColumnCommentSql;
	
	/**
	 * @param createTableSql
	 * @param createTableCommentSql
	 * @param createColumnCommentSql
	 */
	private DbCreateTableEnum(String createTableSql, String createTableCommentSql, String createColumnCommentSql) {
		this.createTableSql = createTableSql;
		this.createTableCommentSql = createTableCommentSql;
		this.createColumnCommentSql = createColumnCommentSql;
	}
	
	public abstract List<String> createTableSql(String tableName, String tableComment, List<FileStructure> structureList);
	
	/**
	 * 根据数据源创建表SQL
	 * @param dsType
	 * @param tableName
	 * @param tableComment
	 * @param structureList
	 * @return
	 */
	public static List<String> buildCreateTableSql(DsTypeEnum dsType, String tableName, String tableComment, List<FileStructure> structureList){
		switch(dsType) {
			case clickhouse:{
				return DbCreateTableEnum.Clickhouse.createTableSql(tableName, tableComment, structureList);
			}
			case mysql:{
				return DbCreateTableEnum.Mysql.createTableSql(tableName, tableComment, structureList);
			}
			case oracle:{
				return DbCreateTableEnum.Oracle.createTableSql(tableName, tableComment, structureList);
			}
			case sqlserver:{
				return DbCreateTableEnum.SqlServer.createTableSql(tableName, tableComment, structureList);
			}
			default:{}
		}
		return null;
	}
}
