/**
 * 
 */
package org.springblade.cube.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.cube.mapper.CubeDataMapper;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeDataPermission;
import org.springblade.cube.model.CubeTableColumn;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.collection.CollectionUtil;

/**
 * @author wookvn
 *
 */
@Service
public class CubeDataService extends ServiceImpl<CubeDataMapper, CubeData> {
	
	@Autowired
	private CubeTableColumnService cubeTableColumnService;
	
	@Autowired
	private CubeDataPermissionService cubeDataPermissionService;
	
	/**
	 * 
	 * @param tableName
	 * @return
	 */
	public List<CubeData> listByTableName(String tableName){
		return baseMapper.listByTableName(tableName);
	}
	
	/**
	 * 查询拥有权限的数据集分类ID
	 * @param queryWrapper
	 * @return
	 */
	public List<Integer> workspaceIdList(Long userId){
		return baseMapper.workspaceIdList(userId);
	}
	
	/**
	 * 
	 * @param cubeData
	 * @param columnList
	 * @return
	 */
	@Transactional
	public R<Integer> doSaveOrUpdate(CubeData cubeData, List<CubeTableColumn> columnList, List<Integer> dataPermIdList, BladeUser bladeUser){
		int rows = cubeData.getId() != null && cubeData.getId() > 0 ? baseMapper.updateById(cubeData) : baseMapper.insert(cubeData);
		if(rows > 0) {
			List<CubeTableColumn> uptList = new ArrayList<CubeTableColumn>();
			List<CubeTableColumn> addList = new ArrayList<CubeTableColumn>();
			for(CubeTableColumn tableColumn : columnList) {
				tableColumn.setCubeDataId(cubeData.getId());
				if(tableColumn.getId() == null || tableColumn.getId() <= 0) {
					tableColumn.setId(null);
					addList.add(tableColumn);
				}else {
					uptList.add(tableColumn);
				}
			}
			//查询旧的ID
			List<Integer> columnIdList = cubeTableColumnService.listObjs(Wrappers.<CubeTableColumn>lambdaQuery().eq(CubeTableColumn::getCubeDataId, cubeData.getId()),t -> Integer.valueOf(t.toString()));
			if(!columnIdList.isEmpty()) {
				//查询需要更新的ID
				List<Integer> uptIdList = uptList.stream().map(CubeTableColumn::getId).collect(Collectors.toList());
				//分析出来需要删除的ID
				List<Integer> removeIdList = columnIdList.stream().filter(f -> !uptIdList.contains(f)).collect(Collectors.toList());
				if(!removeIdList.isEmpty()) {
					cubeTableColumnService.removeByIds(removeIdList);
				}
			}
			//新增和修改
			if(!addList.isEmpty()) cubeTableColumnService.saveBatch(addList);
			if(!uptList.isEmpty()) cubeTableColumnService.updateBatchById(uptList);
			
			//仅增加数据集时处理：权限组处理【前端在编辑时，不显示权限组选择】
			if(CollectionUtil.isNotEmpty(dataPermIdList)) {
				List<CubeDataPermission> entityList = new ArrayList<>();
				for(Integer dataPermId : dataPermIdList) {
					CubeDataPermission cubeDataPerm = new CubeDataPermission();
					cubeDataPerm.setCreateUserId(bladeUser.getUserId());
					cubeDataPerm.setCreateTime(new Date());
					cubeDataPerm.setCubeDataId(cubeData.getId());
//					cubeDataPerm.setCubeWorkspaceId(cubeWorkspaceId);
					cubeDataPerm.setCubePermName("过滤器"+cubeData.getId());
					cubeDataPerm.setDataPermId(dataPermId);
					entityList.add(cubeDataPerm);
				}
				cubeDataPermissionService.saveBatch(entityList);
			}
		}
		return rows > 0 ? R.data(cubeData.getId(), "操作成功") : R.fail("操作失败");
	}
}
