/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.controller;

import java.sql.SQLException;
import java.util.List;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.tool.api.R;
import org.springblade.report.dto.oracle.ReportColumnDTO;
import org.springblade.report.dto.oracle.ReportViewDTO;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportBaseInfoService;
import org.springblade.report.service.IReportBaseService;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.report.service.ITableCategoryRelationService;
import org.springblade.report.service.normal.ReportDatasourcePermissionService;
import org.springblade.system.client.DataPermClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;

/**
 * 产品报表基础信息 控制器
 *
 * @author BladeX
 * @since 2021-03-10
 */
@RestController
@AllArgsConstructor
@RequestMapping("report/baseInfo")
@Api(value = "数据库表基础信息", tags = "数据库表基础信息 ReportBaseController")
public class ReportBaseController extends BladeController {

	@Resource
	private IReportBaseInfoService reportBaseInfoService;

	@Resource
	private IReportBaseService reportBaseService;

	@Resource
	private IReportDatasourceService reportDatasourceService;

	@Resource
	private ITableCategoryRelationService tableCategoryRelationService;

//	@Autowired
//	private IReportDatasourcePermService reportDatasourcePermService;
//
//	@Autowired
//	private IReportDatasourceTablePermService reportDatasourceTablePermService;
	@Autowired
	private DataPermClient dataPermClient;

	@Resource
	private BladeRedis bladeRedis;
	
	@Autowired
	private ReportDatasourcePermissionService reportDatasourcePermissionService;


	/**
	 * 获取报表视图
	 */
	@GetMapping("/listViews")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "获取报表视图", notes = "")
	@Deprecated
	public R<List<ReportViewDTO>> list() {
		List<ReportViewDTO> list = reportBaseService.selectReportViewList();
		return R.data(list);
	}

	/**
	 * 获取报表视图
	 */
	@GetMapping("/listViewsByDataSource")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "根据数据源获取报表视图", notes = "")
//	@Cacheable(cacheNames = "baseInfo", key = "#datasourceId")
	public R<List<ReportViewDTO>> list(HttpServletRequest request, @RequestParam Integer datasourceId) throws ClassNotFoundException, SQLException {
		ReportDatasource datasource = reportDatasourceService.getById(datasourceId);
		if (null == datasource) {
			return R.fail("数据源id错误");
		}
		List<ReportViewDTO> list = reportBaseInfoService.selectReportViewList(datasource);
		if(CollectionUtil.isNotEmpty(list)) { //处理权限组
			reportDatasourcePermissionService.dealTablePermission(getUser().getUserId(), datasourceId, permTableNameSet -> list.removeIf(it -> !permTableNameSet.contains(it.getTableViewName())));
		}
		return R.data(list);
	}


	/**
	 * 获取报表视图的字段信息
	 */
	@GetMapping("/listColumnsByDatasource")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "根据数据源获取报表视图的字段信息", notes = "")
//	@Cacheable(cacheNames = "baseInfo", key = "#reportViewDTO")
	public R<List<ReportColumnDTO>> listColumnsByDatasource(ReportViewDTO reportViewDTO, HttpServletRequest request) throws ClassNotFoundException, SQLException {
		String viewName = reportViewDTO.getTableViewName();
		if (StrUtil.isAllBlank(viewName)) {
			return R.fail("视图名称不可以为空");
		}
		ReportDatasource datasource = reportDatasourceService.getById(reportViewDTO.getDatasourceId());
		if (null == datasource) {
			return R.fail("数据源id错误");
		}
		List<ReportColumnDTO> list = reportBaseInfoService.selectReportColumnList(datasource, viewName);
		return R.data(list);
	}


	@GetMapping("removeBaseInfoCache")
	@ApiOperation(value = "清除baseInfo缓存", notes = "")
	public R removeBaseInfoCache(Integer datasourceId) {
		if (null != datasourceId) {
			bladeRedis.del(bladeRedis.keys("baseInfo::" + datasourceId));
		} else {
			bladeRedis.del(bladeRedis.keys("baseInfo*"));
		}
		return R.success("清除缓存完成");
	}
}
