/**
 * 
 */
package org.springblade.report.enums;

import org.springblade.cube.enums.DbClickHouseEnums;
import org.springblade.cube.enums.DbMysqlEnums;
import org.springblade.cube.enums.DbOracleEnums;
import org.springblade.cube.enums.DbSqlServerEnums;

import com.baomidou.mybatisplus.annotation.EnumValue;

/**
 * @author wookvn
 *
 */
public enum DsTypeEnum {

	mysql(0, "com.mysql.cj.jdbc.Driver", "select 1", DbMysqlEnums.BATCH_SIZE){
		@Override
		public String createUrl(String dsIp, Integer dsPort, String dsDb) {
			return String.format("jdbc:mysql://%s:%s/%s", dsIp, dsPort, dsDb) + "?rewriteBatchedStatements=true&serverTimezone=GMT%2B8";
		}
	},
	
	oracle(1, "oracle.jdbc.OracleDriver", "select 1 from dual", DbOracleEnums.BATCH_SIZE){
		@Override
		public String createUrl(String dsIp, Integer dsPort, String dsDb) {
			return String.format("jdbc:oracle:thin:@%s:%s/%s", dsIp, dsPort, dsDb);
		}
	},
	
	sqlserver(2, "com.microsoft.sqlserver.jdbc.SQLServerDriver", "select 1", DbSqlServerEnums.BATCH_SIZE){
		@Override
		public String createUrl(String dsIp, Integer dsPort, String dsDb) {
			return String.format("jdbc:sqlserver://%s:%s;DatabaseName=%s", dsIp, dsPort, dsDb);
		}
	},
	
	postgresql(3, "org.postgresql.Driver", "select 1", 3000){
		@Override
		public String createUrl(String dsIp, Integer dsPort, String dsDb) {
			return String.format("jdbc:postgresql://%s:%s/%s", dsIp, dsPort, dsDb);
		}
	},
	
	clickhouse(4, "com.clickhouse.jdbc.ClickHouseDriver", "select 1", DbClickHouseEnums.BATCH_SIZE){
		@Override
		public String createUrl(String dsIp, Integer dsPort, String dsDb) {
			return String.format("jdbc:clickhouse://%s:%s/%s", dsIp, dsPort, dsDb);
		}
	},
	//通用版本
	common(-1, "common", "common", 0){
		@Override
		public String createUrl(String dsIp, Integer dsPort, String dsDb) {
			return null;
		}
	};
	
	@EnumValue
	private Integer value;
	private String validationQuery;
	private String driverClassName;
	private int batchSize;  //批量入库数量
	
	private DsTypeEnum(Integer value, String driverClassName, String validationQuery, int batchSize) {
		this.value = value;
		this.driverClassName = driverClassName;
		this.validationQuery = validationQuery;
		this.batchSize = batchSize;
	}
	
	/**
	 * @return the validationQuery
	 */
	public String getValidationQuery() {
		return validationQuery;
	}

	/**
	 * @return the driverClassName
	 */
	public String getDriverClassName() {
		return driverClassName;
	}

	/**
	 * @return the value
	 */
	public Integer getValue() {
		return value;
	}
	
	public int getBatchSize() {
		return batchSize;
	}

	public abstract String createUrl(String dsIp, Integer dsPort, String dsDb);
	
	/**
	 * 解析
	 * @param value
	 * @return
	 */
	public static DsTypeEnum resolve(Integer value) {
		DsTypeEnum[] values = DsTypeEnum.values();
		for(DsTypeEnum dsType : values) {
			if(value != null && dsType.getValue().intValue() == value) {
				return dsType;
			}
		}
		return DsTypeEnum.mysql;
	}
}
