/**
 * 
 */
package org.springblade.report.support;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;

import org.springblade.report.entity.ReportDatasource;

import com.baomidou.dynamic.datasource.provider.AbstractJdbcDataSourceProvider;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DataSourceProperty;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DynamicDataSourceProperties;

import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
public class BladexDataSourceProvider extends AbstractJdbcDataSourceProvider {
	
	private final DynamicDataSourceProperties dynamicDataSourceProperties;
	
	public BladexDataSourceProvider(DynamicDataSourceProperties dynamicDataSourceProperties, String url, String username, String password) {
		super(url, username, password);
		this.dynamicDataSourceProperties = dynamicDataSourceProperties;
	}
	
	@Override
	protected Map<String, DataSourceProperty> executeStmt(Statement statement) throws SQLException {
		log.info("开始初始化动态数据源");
		Map<String, DataSourceProperty> dsMap = new HashMap<String, DataSourceProperty>(16);
		// 构建yml数据源
		Map<String, DataSourceProperty> datasource = dynamicDataSourceProperties.getDatasource();
		if (datasource.size() > 0) {
			dsMap.putAll(datasource);
		}
		
		ResultSet rs = statement.executeQuery("select * from report_datasource where is_deleted = 0");
		while(rs.next()) {
			DataSourceProperty dataSourceProperty = new DataSourceProperty();
			dataSourceProperty.setPoolName(rs.getString("pool_name"));
			dataSourceProperty.setDriverClassName(rs.getString("driver_class_name"));
			dataSourceProperty.setUrl(rs.getString("url"));
			dataSourceProperty.setUsername(rs.getString("ds_user"));
			dataSourceProperty.setPassword(rs.getString("ds_password"));
			//校验连接成功，允许初始化添加到动态数据源
			ReportDatasource reportDs = new ReportDatasource();
			reportDs.setDsUser(dataSourceProperty.getUsername());
			reportDs.setDsPassword(dataSourceProperty.getPassword());
			reportDs.setUrl(dataSourceProperty.getUrl());
			reportDs.setDsType(rs.getInt("ds_type"));
			if(DynamicDataSourceSupport.testDs(reportDs)) { //特殊处理一下数据源
				dataSourceProperty.setDruid(DynamicDataSourceSupport.specialDspDruidConfig(dynamicDataSourceProperties.getDruid(), reportDs.getDsType()));
				dsMap.put(rs.getString("pool_name"), dataSourceProperty);
			}else {
				log.warn("数据源初始化失败|id={},db={},dsType={}", rs.getInt("id"), rs.getString("ds_db"), reportDs.getDsTypeSyl());
			}
		}
		return dsMap;
	}
}
