/**
 * 
 */
package org.springblade.report.support;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.sql.DataSource;

import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.enums.DsTypeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.druid.util.JdbcUtils;
import com.baomidou.dynamic.datasource.DynamicRoutingDataSource;
import com.baomidou.dynamic.datasource.creator.DruidDataSourceCreator;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DataSourceProperty;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DynamicDataSourceProperties;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.druid.DruidConfig;

import cn.hutool.core.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Component
public class DynamicDataSourceSupport {

	@Autowired
	private DynamicRoutingDataSource dynamicRoutingDataSource;

	@Autowired
	private DruidDataSourceCreator dataSourceCreator;
	
	@Autowired
	private DynamicDataSourceProperties dynamicDataSourceProperties;

	/**
	 * 添加数据源
	 * 
	 * @param ds
	 * @return
	 */
	public boolean addDs(ReportDatasource ds) {
		try {
			if (DynamicDataSourceSupport.testDs(ds)) {
				DataSourceProperty dataSourceProperty = new DataSourceProperty();
				dataSourceProperty.setPoolName(ds.getPoolName());
				dataSourceProperty.setUrl(ds.getUrl());
				dataSourceProperty.setUsername(ds.getDsUser());
				dataSourceProperty.setPassword(ds.getDsPassword());
				dataSourceProperty.setDriverClassName(ds.getDriverClassName());
				//特殊处理一下数据源
				dataSourceProperty.setDruid(DynamicDataSourceSupport.specialDspDruidConfig(dynamicDataSourceProperties.getDruid(), ds.getDsType()));
				DataSource dataSource = dataSourceCreator.createDataSource(dataSourceProperty);
				dynamicRoutingDataSource.addDataSource(ds.getPoolName(), dataSource);
				return true;
			}
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
		return false;
	}

	/**
	 * 移除数据源
	 * 
	 * @param ds
	 * @return
	 */
	public boolean removeDs(String poolName) throws Exception {
		dynamicRoutingDataSource.removeDataSource(poolName);
		return true;
	}

	/**
	 * 获取所有数据源
	 * 
	 * @return List<poolName>
	 */
	public List<String> getAllDs() {
		List<String> dsList = new ArrayList<String>();
		Map<String, DataSource> dataSources = dynamicRoutingDataSource.getDataSources();
		if (dataSources != null && !dataSources.isEmpty()) {
			for (Map.Entry<String, DataSource> entry : dataSources.entrySet()) {
				dsList.add(entry.getKey());
			}
		}
		return dsList;
	}

	/**
	 * 测试数据源
	 * 
	 * @param ds
	 * @return
	 */
	public static boolean testDs(ReportDatasource ds) {
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(ds.getDsType());
		
		Connection connection = null;
		Statement statement = null;
		try {
			connection = DynamicDataSourceSupport.createConnection(ds);
			statement = connection.createStatement();
			statement.executeQuery(dsTypeEnum.getValidationQuery());
			return true;
		} catch (Exception e) {
			log.error(e.getMessage());
		} finally {
			JdbcUtils.close(connection);
			JdbcUtils.close(statement);
		}
		return false;
	}
	
	/**
	 * 创建连接
	 * @param ds
	 * @return
	 * @throws SQLException
	 */
	public static Connection createConnection(ReportDatasource ds) throws Exception {
		Properties props = new Properties();
		props.put("user", ds.getDsUser());
		props.put("password", ds.getDsPassword());
		//用于显示字段注释
		props.put("remarks", "true");
//		props.put("useInformationSchema", "true");
		props.put("remarksReporting", "true");
		if(ds.getDsType().intValue() == DsTypeEnum.oracle.getValue()) {
			props.put("oracle.net.CONNECT_TIMEOUT", "1000");
		}else if(ds.getDsType().intValue() == DsTypeEnum.mysql.getValue() || ds.getDsType().intValue() == DsTypeEnum.clickhouse.getValue()){
			props.put("connectTimeout", "1000");
			props.put("nullCatalogMeansCurrent", "true");
		}else if(ds.getDsType().intValue() == DsTypeEnum.postgresql.getValue()) {
			props.put("connectTimeout", "1");
		}else if(ds.getDsType().intValue() == DsTypeEnum.sqlserver.getValue()) {
			props.put("loginTimeout", "1");
		}
		return DriverManager.getConnection(ds.getUrl(), props);
	}
	
	/**
	 * 数据源需要特殊处理
	 * @param dataType
	 */
	public static DruidConfig specialDspDruidConfig(DruidConfig druid, int dataType) {
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(dataType);
		DruidConfig cloneDruidConfig = BeanUtil.copyProperties(druid, DruidConfig.class);
		cloneDruidConfig.setValidationQuery(dsTypeEnum.getValidationQuery());
		return cloneDruidConfig;
	}
}
