package org.springblade.subscribe.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springblade.subscribe.entity.VisualSubscribeNotifyConf;
import org.springblade.subscribe.enums.NotifyType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.stream.Collectors;
@Slf4j
@Component
public class SendEmailService {
    @Autowired
    private VisualSubscribeNotifyConfService notifyConfService;

    @Value("${spring.mail.properties.mail.smtp.timeout}")
    private int timeout;

    @Value("${spring.mail.properties.mail.smtp.writetimeout}")
    private int writetimeout;

    @Value("${spring.mail.properties.mail.smtp.connectiontimeout}")
    private int connectiontimeout;

    @Value("${spring.mail.properties.mail.smtp.socketFactory.class}")
    private String socketFactoryclass;

    @Value("${spring.mail.properties.mail.smtp.socketFactory.fallback}")
    private String fallback;

    @Value("${spring.mail.properties.mail.smtp.socketFactory.port}")
    private String socketFactoryport;




    // 发件人的 邮箱 和 密码/授权码
    // PS: 某些邮箱服务器为了增加邮箱本身密码的安全性，给 SMTP 客户端设置了独立密码（有的邮箱称为“授权码”）
    // 对于开启了独立密码的邮箱, 这里的邮箱密码必需使用这个独立密码（授权码）。
//    private static String myEmailAccount = "发件人邮箱账号";
//    private static String myEmailPassword = "授权码";
//
//    // 发件人邮箱的 SMTP 服务器地址, 可在邮箱设置中查找，需要开启smtp服务
//    // 网易163邮箱的 SMTP 服务器地址为: smtp.163.com
//    private static String smtp = "smtp.163.com";
//
//    // 收件人邮箱
//    private static String receiveMailAccount = "收件人邮箱账号";

    /**
     * 发送邮件
     *
     * @param emailContent
     */
    public void senEmail(String receiveMailAccount,String subject,String emailContent) {
        try {
            //查询水印配置
            QueryWrapper<VisualSubscribeNotifyConf> wrapper =new QueryWrapper<>();
            wrapper.lambda().eq(VisualSubscribeNotifyConf::getNotifyType, NotifyType.email);
            List<VisualSubscribeNotifyConf> watermarkList = notifyConfService.list(wrapper);
            Map<String, String> emailMap = watermarkList.stream().collect(Collectors.toMap(VisualSubscribeNotifyConf::getAttriKey, VisualSubscribeNotifyConf::getAttriValue));
            String username = emailMap.get("username");
            String password = emailMap.get("password");
            String smtp = emailMap.get("smtp");
            log.info("username:"+username);
            log.info("password:"+password);
            log.info("smtp:"+smtp);

            // 1. 创建参数配置, 用于连接邮件服务器的参数配置
            Properties props = new Properties();
            // 使用的协议（JavaMail规范要求）
            props.setProperty("mail.transport.protocol", "smtp");
            // 发件人的邮箱的 SMTP 服务器地址
            props.setProperty("mail.smtp.host", smtp);
            // 需要请求认证
            props.setProperty("mail.smtp.auth", "true");
            props.put("mail.smtp.port", "465");
            props.put("mail.smtp.timeout", timeout);
            props.put("mail.smtp.connectiontimeout", connectiontimeout);
            props.put("mail.smtp.writetimeout", writetimeout);
            props.put("mail.smtp.socketFactory.port", socketFactoryport);
            props.put("mail.smtp.socketFactory.class", socketFactoryclass);
            props.put("mail.smtp.socketFactory.fallback",fallback);

            // 2. 根据配置创建会话对象, 用于和邮件服务器交互
            Session session = Session.getInstance(props);
            // 设置为debug模式, 可以查看详细的发送 log
            session.setDebug(false);
            // 3. 创建一封邮件    session、发件人、收件人、邮件内容,主题
            MimeMessage message = createMimeMessage(session, username, receiveMailAccount, emailContent,subject);
            // 4. 根据 Session 获取邮件传输对象
            Transport transport = session.getTransport();

            // 5. 使用 邮箱账号 和 密码/授权码 连接邮件服务器, 这里认证的邮箱必须与 message 中的发件人邮箱一致, 否则报错
            transport.connect(username, password);

            // 6. 发送邮件, 发到所有的收件地址, message.getAllRecipients() 获取到的是在创建邮件对象时添加的所有收件人, 抄送人, 密送人
            transport.sendMessage(message, message.getAllRecipients());
            // 7. 关闭连接
            transport.close();
            log.info("邮件发送成功，接收邮箱为："+receiveMailAccount);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 创建邮件
     *
     * @param session
     * @param myEmailAccount
     * @param receiveMailAccount
     * @return
     */
    private MimeMessage createMimeMessage(Session session, String myEmailAccount, String receiveMailAccount, String content,String subject) {
        try {
            // 1. 创建一封邮件
            MimeMessage mimeMessage = new MimeMessage(session);
            // 2. From: 发件人   emailUserName、昵称、编码格式
            mimeMessage.setFrom(new InternetAddress(myEmailAccount, "几何BI" + "<" + myEmailAccount + ">", "UTF-8"));

            // 3. To: 收件人（可以增加多个收件人）
            mimeMessage.setRecipient(MimeMessage.RecipientType.TO, new InternetAddress(receiveMailAccount, "收件人名称", "UTF-8"));
            //多个收件人
            /**String[] split = receiveMailAccount.split(",");
             for(int i = 0; i < split.length; i++){
             mimeMessage.addRecipient(MimeMessage.RecipientType.TO, new InternetAddress(split[i]));
             }*/

            // 4. Subject: 邮件主题
            mimeMessage.setSubject(subject, "UTF-8");
            // 5. Content: 邮件正文（可以使用html标签）
            mimeMessage.setContent(content, "text/html;charset=UTF-8");
            // 6. 设置发件时间
            mimeMessage.setSentDate(new Date());

            // 7. 保存设置
            mimeMessage.saveChanges();

            return mimeMessage;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

}