package org.springblade.visual.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.ApiOperation;
import org.codehaus.groovy.syntax.TokenUtil;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.system.client.UserClient;
import org.springblade.system.client.model.UserModel;
import org.springblade.visual.entity.VisualLoginFreeLink;
import org.springblade.visual.service.VisualLoginFreeLinkService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @author cyc
 *
 */
@RestController
@RequestMapping("/visual/test")
public class VisualCreateUrlController {

    @Autowired
    private UserClient userClient;

    @Autowired
    private VisualLoginFreeLinkService loginFreeLinkService;

    @Value("${blade.loginFreeDomain}")
    private String domain;


    /**
     * 免登录链接功能
     */
    @GetMapping("/getUrlByShortLink")
    @ApiOperation(value = "免登录链接功能")
    @ApiOperationSupport(order = 1)
    public R<String> getUrlByShortLink(String shortLink) throws ParseException {
        VisualLoginFreeLink loginFreeLink =loginFreeLinkService.getOne(new LambdaQueryWrapper<VisualLoginFreeLink>()
                .eq(VisualLoginFreeLink::getShortLink,domain+"/#/third/"+shortLink)
                .eq(VisualLoginFreeLink::getStatus,1)

        );
        if (loginFreeLink != null){
            //判断是否过期
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            Date validDate = format.parse( loginFreeLink.getValidDate());
            long  dateTime= validDate.getTime();
            long nowTime = System.currentTimeMillis();
            if (nowTime >= dateTime){
                return R.fail("该链接授权已过期!");
            }
            String accessToken = userClient.genAccessToken(loginFreeLink.getUserId());
            return R.data(String.format("%s?token=%s",loginFreeLink.getRealUrl(),accessToken));
        }else {
            return R.fail("该链接未被授权或者未上线!");
        }
    }




    /**
     * 验证身份，通过token获取真实链接
     */
    @GetMapping("/getUrlValid")
    @ApiOperation(value = "创建链接")
    @ApiOperationSupport(order = 2)
    public R<String> getUrlValid(String userId,String token,String shortLink,long expiresTime) throws ParseException {
        UserModel userModel = userClient.getByThirdUserId(userId);
        VisualLoginFreeLink loginFreeLink =loginFreeLinkService.getOne(new LambdaQueryWrapper<VisualLoginFreeLink>()
                .eq(VisualLoginFreeLink::getShortLink,shortLink)
                .eq(VisualLoginFreeLink::getStatus,1)
        );
        if (userModel!=null && loginFreeLink!= null){
            //判断是否过期
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            Date validDate = format.parse( loginFreeLink.getValidDate());
            long dateTime= validDate.getTime();
            long nowTime = System.currentTimeMillis();
            if (nowTime >= dateTime){
                return R.fail("该链接授权已过期!");
            }
            //保存三方token
            loginFreeLink.setUserToken(token);
            //生成本系统token
            String genToken = userClient.genToken(userModel.getId(), String.valueOf(expiresTime*1000));
            loginFreeLink.setAccessToken(genToken);
            loginFreeLinkService.updateById(loginFreeLink);
            return R.data(String.format("%s?token=%s",shortLink,genToken));
        }else {
            return R.fail("该用户没有权限");
        }

    }
    /**
     * 不需要验证身份
     */
    @GetMapping("/getUrlNoValid")
    @ApiOperation(value = "创建链接")
    @ApiOperationSupport(order = 2)
    public R<String> getUrlNoValid(String userId,String shortLink,long expiresTime) throws ParseException {
        UserModel userModel = userClient.getByThirdUserId(userId);
        VisualLoginFreeLink loginFreeLink =loginFreeLinkService.getOne(new LambdaQueryWrapper<VisualLoginFreeLink>()
                .eq(VisualLoginFreeLink::getShortLink,shortLink)
                .eq(VisualLoginFreeLink::getStatus,1)
        );
        if (userModel!=null && loginFreeLink!= null){
            //判断是否过期
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            Date validDate = format.parse( loginFreeLink.getValidDate());
            long dateTime= validDate.getTime();
            long nowTime = System.currentTimeMillis();
            if (nowTime >= dateTime){
                return R.fail("该链接授权已过期!");
            }
            //生成本系统token
            String genToken = userClient.genToken(userModel.getId(), String.valueOf(expiresTime*1000));
            loginFreeLink.setAccessToken(genToken);
            loginFreeLinkService.updateById(loginFreeLink);
            return R.data(String.format("%s?token=%s",loginFreeLink.getRealUrl(),genToken));
        }else {
            return R.fail("该用户没有权限");
        }

    }

    /**
     * 判断token有效性
     */
    @GetMapping("/checkToken")
    @ApiOperation(value = "创建链接")
    @ApiOperationSupport(order = 2)
    public R<String> checkToken(String token,String shortLink) throws ParseException {
        VisualLoginFreeLink loginFreeLink =loginFreeLinkService.getOne(new LambdaQueryWrapper<VisualLoginFreeLink>()
                .eq(VisualLoginFreeLink::getUserToken,token)
                .eq(VisualLoginFreeLink::getShortLink,shortLink)
        );
        if (loginFreeLink!= null){
            return R.data(loginFreeLink.getRealUrl());
        }else {
            return R.fail("该用户没有权限");
        }
    }
}
