/**
 * 
 */
package org.springblade.visual.controller;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.cube.enums.DbClickHouseEnums;
import org.springblade.cube.enums.DbMysqlEnums;
import org.springblade.cube.enums.DbOracleEnums;
import org.springblade.cube.enums.DbSqlServerEnums;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@RestController
@RequestMapping("visual/dynamicData")
@Api(value = "动态数据接口", tags = "动态数据接口")
public class VisualDynamicDataController extends BladeController {
	
	@Autowired
	private DynamicTableService dynamicTableService;
	
	@Autowired
	private IReportDatasourceService reportDatasourceService;
	
	@Autowired
	private CubeDataService cubeDataService;
	
	private final static int DEFAULT_LIMIT = 200;
	
	/**
	 * 获取枚举值列表
	 * @return
	 */
	@GetMapping("/enumValueList")
	@ApiOperation(value = "获取枚举数据", notes = "返回数组")
	public R<List<Map<String, Object>>> enumValueList(Integer cubeDataId, String tableName, String columnName,
			Integer visualId, Integer componentId){
		CubeData cubeData = cubeDataService.getById(cubeDataId);
		if(cubeData == null) {
			return R.fail("数据集不存在");
		}
		String sql = String.format("select %s, count(1) cnt from %s group by %s order by cnt desc", 
				columnName, tableName, columnName);
		// 前提:查询数据源
		ReportDatasource reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
		if (reportDatasource == null) {
			return R.fail("数据源不存在");
		}
		log.info("enumValueList|tableName={},visualId={},componentId={}", tableName, visualId, componentId);
		return R.data(dynamicTableService.selectPageList(new Page<>(0, DEFAULT_LIMIT).setSearchCount(false), reportDatasource, sql));
	}
	
	/**
	 * 校验表记录数 [仅Oracle和Mysql校验]
	 * @return
	 */
	@GetMapping("/checkTableUseEnum")
	@ApiOperation(value = "校验表是否适用于动态枚举")
	public R<String> checkTableDataSize(Integer cubeDataId, String tableName, String columnName) {
		CubeData cubeData = cubeDataService.getById(cubeDataId);
		if(cubeData == null) {
			return R.fail("数据集不存在");
		}
		ReportDatasource reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
		if (reportDatasource == null) {
			return R.fail("数据源不存在");
		}
		String checkSql = null;
		Integer maxLength = 0;
		switch(reportDatasource.getDsTypeSyl()) {
			case oracle:{
				checkSql = String.format(DbOracleEnums.CHECK_TABLE_DATASIZE_SQL_FORMAT, tableName, DbOracleEnums.DEFAULT_MAX_SIZE);
				maxLength = DbOracleEnums.DEFAULT_MAX_SIZE;
				break;
			}
			case mysql:{
				checkSql = String.format(DbMysqlEnums.CHECK_TABLE_DATASIZE_SQL_FORMAT, tableName, DbMysqlEnums.DEFAULT_MAX_SIZE);
				maxLength = DbMysqlEnums.DEFAULT_MAX_SIZE;
				break;
			}
			case sqlserver:{
				checkSql = String.format(DbSqlServerEnums.CHECK_TABLE_DATASIZE_SQL_FORMAT, tableName);
				maxLength = DbClickHouseEnums.DEFAULT_MAX_SIZE;
				break;
			}
			case clickhouse:{
				checkSql = String.format(DbClickHouseEnums.CHECK_TABLE_DATASIZE_SQL_FORMAT, tableName);
				maxLength = DbClickHouseEnums.DEFAULT_MAX_SIZE;
				break;
			}
			default:{}
		}
		if(checkSql == null || maxLength == 0) {
			return R.fail("不支持的数据源");
		}
		Map<String, Object> resultMap = dynamicTableService.selectOne(reportDatasource, checkSql);
		if(resultMap != null && !resultMap.isEmpty()) {
			Entry<String, Object> entry = resultMap.entrySet().iterator().next();
			long cnt = Func.toLong(entry.getValue().toString(), 0L);
			if(cnt >= maxLength) {
				return R.fail("数据量太大，不能使用<原始数据聚合>。");
			}
		}
		return R.success("可以使用<原始数据聚合>");
	}
}
