package org.springblade.visual.controller;


import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.xxl.job.core.biz.AdminBiz;
import com.xxl.job.core.biz.client.AdminBizClient;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.usergroup.entity.GroupClassify;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualLoginFreeLink;
import org.springblade.visual.service.VisualDataService;
import org.springblade.visual.service.VisualLoginFreeLinkService;
import org.springblade.visual.service.VisualShortCodeGeneratorService;
import org.springblade.visual.utils.Number62;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;


/**
 * @author cyc
 *
 */
@RestController
@RequestMapping("visual/longinfreelink")
@Api(value = "免登陆链接接口", tags = "免登陆链接接口")
public class VisualLoginFreeLinkController extends BladeController {

    @Autowired
    private VisualLoginFreeLinkService loginFreeLinkService;

    @Autowired
    private VisualShortCodeGeneratorService codeGeneratorService;

    @Autowired
    private VisualDataService visualDataService;

    @Value("${blade.loginFreeDomain}")
    private String domain;

    /**
     * 列表
     */
    @GetMapping("/list")
    @ApiOperation(value = "免登陆链接列表")
    @ApiOperationSupport(order = 1)
    public R<IPage<VisualLoginFreeLink>> list(VisualLoginFreeLink visualLoginFreeLink, Query query){
        QueryWrapper queryWrapper = Condition.getQueryWrapper(visualLoginFreeLink);
        if (StrUtil.isNotBlank(visualLoginFreeLink.getVisualDataTitle())) {
            queryWrapper.like("visual_data_title", "%" + visualLoginFreeLink.getVisualDataTitle() + "%");
        }
        queryWrapper.orderByDesc("id");
        return R.data(loginFreeLinkService.page(Condition.getPage(query), queryWrapper));
    }


    /**
     * 单条数据查询接口
     * @param id
     * @return
     */
    @GetMapping("/detail")
    @ApiOperation(value = "根据ID获取免登陆链接", httpMethod = "GET")
    @ApiOperationSupport(order = 2)
    public R<VisualLoginFreeLink> info(@RequestParam Integer id) {
        return R.data(loginFreeLinkService.getById(id));
    }


    /**
     * 	删除
     * @param ids
     * @return
     */
    @PostMapping("/remove")
    @ApiOperation(value = "根据ID数组删除免登陆链接(多个逗号分割)", httpMethod = "POST")
    @ApiOperationSupport(order = 3)
    public R<String> doRemove(@ApiParam(value = "主键", required = true) @RequestParam String ids) {
        return R.status(loginFreeLinkService.removeByIds(Func.toIntList(ids)));
    }

    /**
     * 保存
     */
    @PostMapping("/save")
    @ApiOperationSupport(order = 4)
    @ApiOperation(value = "新增 save", notes = "传入visualLoginFreeLink")
    public R<Integer> save(@Valid @RequestBody VisualLoginFreeLink visualLoginFreeLink) {
        VisualData visualData = visualDataService.getById(visualLoginFreeLink.getVisualDataId());
        if (visualData == null){
            return R.fail("大屏不存在！");
        }
        if (visualLoginFreeLink.getId() == null){
            visualLoginFreeLink.setCreateBy(getUser().getNickName());
            visualLoginFreeLink.setCreateTime(new Date());
            visualLoginFreeLink.setCreateUserId(getUser().getUserId());
            visualLoginFreeLink.setStatus(0);
            visualLoginFreeLink.setVisualDataTitle(visualData.getTitle());
            if(visualLoginFreeLink.getLinkType() == 0){//免登录
                visualLoginFreeLink.setShortLink(domain+"/#/third/"+getCode());
            }else {
                visualLoginFreeLink.setShortLink(domain+"/"+getCode());
            }
            visualLoginFreeLink.setRealUrl(String.format("%s%s/%s",domain+"/#/newbi/mobile/",visualData.getCategory(),visualData.getId()));
            loginFreeLinkService.save(visualLoginFreeLink);
        }else {
            visualLoginFreeLink.setUpdateBy(getUser().getNickName());
            visualLoginFreeLink.setUpdateTime(new Date());
            visualLoginFreeLink.setUpdateUserId(getUser().getUserId());
            loginFreeLinkService.updateById(visualLoginFreeLink);
        }
        return R.data(visualLoginFreeLink.getId());
    }

    /**
     * 生成短链接编码
     */
    private String getCode() {
        Long  codeNum = codeGeneratorService.getCodeNum();
        String shortCode = Number62.encode(codeNum.longValue());
        return shortCode;
    }





}
