/**
 * 
 */
package org.springblade.visual.controller;

import java.util.Arrays;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.visual.entity.VisualPowerPointPermission;
import org.springblade.visual.service.VisualPowerPointPermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**
 *	幻灯片权限组 操作处理
 *
 * @author wookvn
 * @date   2023-02-04
 */
@ApiIgnore
@RestController
@RequestMapping("/powerPointPermission")
@Api(value = "幻灯片权限组相关接口", tags = "幻灯片权限组接口")
public class VisualPowerPointPermissionController extends BladeController {

	@Autowired
	private VisualPowerPointPermissionService visualPowerPointPermissionService;
	
	/**
	 * 	列表数据
	 * @param visualPowerPointPermission
	 * @return
	 */
	@GetMapping("/page")
	@ApiOperation(value = "分页查询幻灯片权限组", httpMethod = "GET")
	public R<IPage<VisualPowerPointPermission>> pageList(VisualPowerPointPermission visualPowerPointPermission, Query query){
		QueryWrapper<VisualPowerPointPermission> queryWrapper = Condition.getQueryWrapper(visualPowerPointPermission);
		return R.data(visualPowerPointPermissionService.page(Condition.getPage(query), queryWrapper));
	}
	
	/**
	 * 单条数据查询接口
	 * @param id
	 * @return
	 */
	@GetMapping("/info")
	@ApiOperation(value = "根据ID获取幻灯片权限组", httpMethod = "GET")
	public R<VisualPowerPointPermission> info(@RequestParam Integer id) {
		return R.data(visualPowerPointPermissionService.getById(id));
	}
	
	/**
	 *	修改数据接口
	 * @param visualPowerPointPermission
	 * @return
	 */
	@PostMapping("/submit")
	@ApiOperation(value = "增加或修改幻灯片权限组", httpMethod = "POST")
	public R<String> doUpdate(@Validated @RequestBody VisualPowerPointPermission visualPowerPointPermission) {
		return R.status(visualPowerPointPermissionService.saveOrUpdate(visualPowerPointPermission));
	}
	
	/**
	 * 	删除数据接口
	 * @param id
	 * @return
	 */
	@PostMapping("/delete")
	@ApiOperation(value = "根据ID删除幻灯片权限组", notes = "多个逗号分割", httpMethod = "POST")
	public R<String> doDelete(@RequestParam Integer[] id) {
		return R.status(visualPowerPointPermissionService.removeByIds(Arrays.asList(id)));
	}
	
}