/**
 * 
 */
package org.springblade.visual.enums;

import java.util.List;
import java.util.stream.Collectors;

import org.springblade.cube.enums.ViewColumnTypeEnum;
import org.springblade.report.enums.DsTypeEnum;

/**
 * @author wookvn
 *
 */
public enum CommonFunctionEnum implements InterfaceSqlFunction {

	eq {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			return values != null && values.size() >= 1 ? String.format("%s=" + format(columnType, dsTypeEnum, values.get(0)), columnSql) : "";
		}
	},

	ne {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			return values != null && values.size() >= 1 ? String.format("%s!=" + format(columnType, dsTypeEnum, values.get(0)), columnSql) : "";
		}
	},

	lt {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			return values != null && values.size() >= 1 ? String.format("%s<" + format(columnType, dsTypeEnum, values.get(0)), columnSql) : "";
		}
	},

	le {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			return values != null && values.size() >= 1 ? String.format("%s<=" + format(columnType, dsTypeEnum, values.get(0)), columnSql) : "";
		}
	},

	gt {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			return values != null && values.size() >= 1 ? String.format("%s>" + format(columnType, dsTypeEnum, values.get(0)), columnSql) : "";
		}
	},

	ge {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			return values != null && values.size() >= 1 ? String.format("%s>=" + format(columnType, dsTypeEnum, values.get(0)), columnSql) : "";
		}
	},

	in {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			if(values != null && values.size() >= 1) {
				List<String> _values = values.stream().map(f -> String.format("%s'%s'", sqlServerNString(f, dsTypeEnum), f.toString())).collect(Collectors.toList());
				return String.format("%s in (%s)", columnSql, String.join(",", _values));
			}
			return "";
		}
	},

	notIn {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			if(values != null && values.size() >= 1) {
				List<String> _values = values.stream().map(f -> String.format("%s'%s'", sqlServerNString(f, dsTypeEnum), f.toString())).collect(Collectors.toList());
				return String.format("%s not in (%s)", columnSql, String.join(",", _values));
			}
			return "";
		}
	},

	like {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			return values != null && values.size() >= 1 ? String.format("%s like %s'%%%s%%'", columnSql, sqlServerNString(values.get(0), dsTypeEnum), values.get(0)) : "";
		}
	},
	
	notLike {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			return values != null && values.size() >= 1 ? String.format("%s not like %s'%%%s%%'", columnSql, sqlServerNString(values.get(0), dsTypeEnum), values.get(0)) : "";
		}
	},

	leftLike {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			return values != null && values.size() >= 1 && columnType == ViewColumnTypeEnum.STRING ? String.format("%s like %s'%%%s'", columnSql, sqlServerNString(values.get(0), dsTypeEnum), values.get(0)) : "";
		}
	},

	rightLike {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			return values != null && values.size() >= 1 && columnType == ViewColumnTypeEnum.STRING ? String.format("%s like %s'%s%%'", columnSql, sqlServerNString(values.get(0), dsTypeEnum), values.get(0)) : "";
		}
	},
	
	between {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			return values != null && values.size() >= 2 ? String.format("(%s between %s and %s)", columnSql, format(columnType, dsTypeEnum, values.get(0)) , format(columnType, dsTypeEnum, values.get(1))) : "";
		}
	},
	
	notnull {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			return String.format("%s is not null", columnSql);
		}
	},
	
	isnull {
		@Override
		public String handle(String columnSql, ViewColumnTypeEnum columnType, List<Object> values, DsTypeEnum dsTypeEnum) {
			return String.format("%s is null", columnSql);
		}
	}
}
