/**
 * 
 */
package org.springblade.visual.service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springblade.core.tool.api.R;
import org.springblade.cube.dto.CubeTableName;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.CubeDataService;
import org.springblade.visual.dto.BloodRelationDto;
import org.springblade.visual.mapper.BloodRelationMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Service
public class BloodRelationService {

	@Autowired
	private BloodRelationMapper bloodRelationMapper;

	@Autowired
	private CubeDataService cubeDataService;
	
	/**
	 * 
	 * @param cubeDataId
	 * @param userId
	 * @return
	 */
	public R<BloodRelationDto> relationTree(Integer cubeDataId, Long userId) {
		CubeData cubeData = cubeDataService.getById(cubeDataId);
		if (cubeData == null) {
			return R.fail("数据集不存在");
		}
		List<BloodRelationDto> dataList = new ArrayList<>();
		_buildTreeList(dataList, Arrays.asList(cubeDataId), userId);
		//取出第一条
		BloodRelationDto resultData = dataList.get(0);
		List<CubeTableName> tableLinkJoinList = JSON.parseArray(cubeData.getCubeTableNames(), CubeTableName.class);
		// 添加表关系
		for (CubeTableName cubeTableName : tableLinkJoinList) {
			String tableName = StrUtil.isNotBlank(cubeTableName.getTableComment()) ? cubeTableName.getTableComment()
					: cubeTableName.getTableName();
			resultData.getChildren().add(new BloodRelationDto("0", tableName, "table"));
		}
		return R.data(resultData);
	}

	/**
	 * 
	 * @param tableName
	 * @param userId
	 * @return
	 */
	public R<BloodRelationDto> relationTree(String tableName, Long userId) {
		// 「表的中文名称：前端直接显示即可」{id:0, children: [{id: 1, children:[{}]}, {id: 2, children:
		// []}, {id: 3, children:[{}]}]}
		BloodRelationDto resultData = new BloodRelationDto("0", tableName, "table");
		List<BloodRelationDto> dealList = new ArrayList<>();
		resultData.setChildren(dealList);
		
		//根据表名查询数据集列表
		List<CubeData> cubeDataList = cubeDataService.listByTableName(tableName);
		if(CollectionUtil.isEmpty(cubeDataList)) {
			return R.data(resultData);
		}
		_buildTreeList(dealList, cubeDataList.stream().map(CubeData::getId).collect(Collectors.toList()), userId);
		return R.data(resultData);
	}
	
	/**
	 * 构建数据
	 * @param dealList
	 * @param cubeDataIdList
	 * @param userId
	 */
	private void _buildTreeList(List<BloodRelationDto> dealList, List<Integer> cubeDataIdList, Long userId) {
		List<BloodRelationDto> dataList = new ArrayList<>();
		String cubeDataIds = cubeDataIdList.stream().map(t -> t.toString()).collect(Collectors.joining(","));
		// 添加报告相关的树 [{id: 1, children:[{}]}, {id: 2, children: null}]
		try {
			List<BloodRelationDto> reportTreeList = bloodRelationMapper.reportTreeList(userId, cubeDataIds);
			if (CollectionUtil.isNotEmpty(reportTreeList)) {
				dataList.addAll(reportTreeList);
			}
		} catch (Exception e) {
			log.error("报告/大屏/组件树结构获取失败|message="+e.getMessage(), e);
		}
		// 添加分类相关的树 [{id: 2, children:null}, {id: 3, children:[{}]}]
		try {
			List<BloodRelationDto> cateTreeList = bloodRelationMapper.cateTreeList(userId, cubeDataIds);
			if (CollectionUtil.isNotEmpty(cateTreeList)) {
				dataList.addAll(cateTreeList);
			}
		} catch (Exception e) {
			log.error("大屏/组件/大屏分类树结构获取失败|message="+e.getMessage(), e);
		}
		// 添加预警器树
		try {
			List<BloodRelationDto> warningTreeList = bloodRelationMapper.warningTreeList(cubeDataIds);
			if(CollectionUtil.isNotEmpty(warningTreeList)) {
				dataList.addAll(warningTreeList);
			}
		} catch (Exception e) {
			log.error("智能预警树结构获取失败|message="+e.getMessage(), e);
		}
		//添加自定义人群树
		try {
			List<BloodRelationDto> defineGroupTreeList = bloodRelationMapper.defineGroupTreeList(cubeDataIds);
			if(CollectionUtil.isNotEmpty(defineGroupTreeList)) {
				dataList.addAll(defineGroupTreeList);
			}
		} catch (Exception e) {
			log.error("自定义人群树结构获取失败|message="+e.getMessage(), e);
		}
		//添加表单树
		try {
			List<BloodRelationDto> formModelTreeList = bloodRelationMapper.formModelTreeList(cubeDataIds);
			if(CollectionUtil.isNotEmpty(formModelTreeList)) {
				dataList.addAll(formModelTreeList);
			}
		} catch (Exception e) {
			log.error("表单树结构获取失败|message="+e.getMessage(), e);
		}
		//报告和分类的数据列表，根据数据集ID聚合重新聚合数据结果{原因：分类和报告查询的数据集有可能相同，需要重新规整}
		Map<String, BloodRelationDto> dataMap = new HashMap<>();
		for (BloodRelationDto bloodRelationDto : dataList) {
			if (!dataMap.containsKey(bloodRelationDto.getId())) {
				BloodRelationDto data = new BloodRelationDto();
				data.setId(bloodRelationDto.getId());
				data.setLabel(bloodRelationDto.getLabel());
				data.setModel(bloodRelationDto.getModel());
				data.setModule(bloodRelationDto.getModule());
				data.setChildren(new ArrayList<>());
				dataMap.put(bloodRelationDto.getId(), data);
			}
			if(bloodRelationDto.getChildren() != null) {
				dataMap.get(bloodRelationDto.getId()).getChildren().addAll(bloodRelationDto.getChildren());
			}
		}
		dataMap.forEach((k, v) -> dealList.add(v));
	}

}
