/**
 * 
 */
package org.springblade.visual.service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.utils.Func;
import org.springblade.visual.entity.VisualPowerPointInfo;
import org.springblade.visual.mapper.VisualPowerPointInfoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

/**
 *	幻灯片 服务处理
 *
 * @author wookvn
 * @date   2023-02-04
 */
@Service
public class VisualPowerPointInfoService extends ServiceImpl<VisualPowerPointInfoMapper, VisualPowerPointInfo>{
	
	@Autowired
	private VisualDataService visualDataService;
	
	/**
	 * 根据ID删除一个幻灯片
	 * @param id
	 * @return
	 */
	@Transactional
	public boolean doRemove(Integer id) {
		VisualPowerPointInfo powerPointInfo = baseMapper.selectById(id);
		if(StrUtil.isNotBlank(powerPointInfo.getVisualIds())) {
			List<Integer> visualIdList = new LinkedList<Integer>(Func.toIntList(powerPointInfo.getVisualIds()));
			visualDataService.doRemove(visualIdList);
		}
		return baseMapper.deleteById(id) > 0;
	}
	
	/**
	 * 更新幻灯片的大屏列表
	 * @param ppId
	 * @param visualIds
	 * @return
	 */
	@Transactional
	public boolean doUpdateVisual(Integer ppId, String visualIds, BladeUser bladeUser) {
		//查询已存在的大屏ID
		VisualPowerPointInfo powerPointInfo = baseMapper.selectById(ppId);
		List<Integer> dbVisualIdList = new ArrayList<Integer>(Func.toIntList(powerPointInfo.getVisualIds()));
		List<Integer> newVisualIdList = new ArrayList<Integer>(Func.toIntList(visualIds));
		//过滤出来需要删除和新增的
		List<Integer> delVisualIdList = dbVisualIdList.stream().filter(item -> !newVisualIdList.contains(item)).collect(Collectors.toList());
		List<Integer> addVisualIdList = newVisualIdList.stream().filter(item -> !dbVisualIdList.contains(item)).collect(Collectors.toList());
		if(CollectionUtil.isNotEmpty(addVisualIdList)) {
			visualDataService.doCopyPowerPoint(addVisualIdList, ppId, bladeUser);
		}
		if(CollectionUtil.isNotEmpty(delVisualIdList)) {
			visualDataService.doRemove(delVisualIdList);
		}
		//更新幻灯片的大屏列表
		VisualPowerPointInfo entity = new VisualPowerPointInfo();
		entity.setId(ppId);
		entity.setVisualIds(visualIds);
		return baseMapper.updateById(entity) > 0;
	}
	
	/**
	 * 删除单张大屏
	 * @param ppId
	 * @param visualId
	 * @return
	 */
	@Transactional
	public boolean doRemoveVisual(Integer ppId, Integer visualId) {
		VisualPowerPointInfo powerPointInfo = baseMapper.selectById(ppId);
		if(StrUtil.isNotBlank(powerPointInfo.getVisualIds())) {
			List<String> visualIdList = new LinkedList<String>(Func.toStrList(powerPointInfo.getVisualIds()));
			boolean ret = visualIdList.removeIf(item -> item.equals(visualId.toString()));
			if(ret) {
				//删除大屏
				visualDataService.doRemove(Arrays.asList(visualId));
				//更新幻灯片
				VisualPowerPointInfo entity = new VisualPowerPointInfo();
				entity.setId(ppId);
				entity.setVisualIds(String.join(",", visualIdList));
				baseMapper.updateById(entity);
			}
		}
		return true;
	}
}