-- 20221010 - 王坤 - 函数表修改
ALTER TABLE `cube_datasource_function` 
MODIFY COLUMN `ds_type` tinyint(4) NOT NULL COMMENT '数据源类型(0-mysql/1-oracle/2-sqlserver/3-postgresql/4-clickhouse/-1通用)' AFTER `method`,
ADD COLUMN `func_type` tinyint(4) NOT NULL DEFAULT 0 COMMENT '函数类型(0-数据库自带函数,1-自定义解析函数)' AFTER `sort_num`,
ADD COLUMN `func_sql` varchar(1000) NULL COMMENT '自定义函数sql片段' AFTER `func_type`,
ADD COLUMN `func_attr` tinyint(4) NULL COMMENT '0-聚合函数\n1-统计函数\n2-日期和时间\n3-字符串函数\n4-逻辑函数\n5-数值函数\n6-数组函数\n7-自定义函数\n99-其他' AFTER `func_sql`,
ADD COLUMN `method_name` varchar(50) NULL COMMENT '函数名称' AFTER `func_attr`,
ADD COLUMN `sub_query_status` tinyint(4) NOT NULL DEFAULT 0 COMMENT '是否包含子查询(1-是/0-否)' AFTER `method_name`;

-- 20221010 - 王坤 - 数据集字段表添加是否包含聚合函数字段
ALTER TABLE `cube_table_column` 
ADD COLUMN `virtual_aggregation_status` tinyint(4) NOT NULL DEFAULT 0 COMMENT '虚拟字段是否包含聚合函数' AFTER `create_user_id`,
ADD COLUMN `virtual_only_zhibiao_status` tinyint(4) NOT NULL DEFAULT 0 COMMENT '虚拟字段是否仅用于指标列查询' AFTER `virtual_aggregation_status`;

-- 20221011 - 王坤 - 添加权限组
drop table if exists `cube_data_permission`;
CREATE TABLE `cube_data_permission` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `cube_perm_name` varchar(100) NOT NULL COMMENT '数据集权限名称',
  `cube_data_id` int(11) NOT NULL COMMENT '数据集ID',
  `data_perm_id` int(11) NOT NULL COMMENT '权限组ID',
  `filter_json` text DEFAULT NULL COMMENT '过滤器JSON',
  `filter_sql` varchar(1000) DEFAULT NULL COMMENT '[行权限]过滤器SQL',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  `create_user_id` bigint(20) NOT NULL COMMENT '创建人',
  `exclude_column_ids` varchar(1000) DEFAULT NULL COMMENT '[列权限]不包含的字段ID,逗号隔开',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `cube_data_id` (`cube_data_id`,`data_perm_id`) USING BTREE
) ENGINE=InnoDB CHARSET=utf8mb4 COMMENT='数据集权限';

-- 20221024 - 王坤 - 添加订阅服务表和订阅日志表、三方配置表
drop table if exists `visual_subscribe`;
CREATE TABLE `visual_subscribe` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `sub_title` varchar(100) NOT NULL COMMENT '订阅名称/标题',
  `data_type` tinyint(4) NOT NULL COMMENT '订阅内容类型',
  `data_id` int(11) NOT NULL COMMENT '订阅内容ID',
  `trigger_period` tinyint(4) NOT NULL COMMENT '触发周期',
  `trigger_week_value` tinyint(4) DEFAULT NULL COMMENT '触发周几',
  `trigger_day_value` tinyint(4) DEFAULT NULL COMMENT '触发哪一天',
  `trigger_hour` tinyint(4) NOT NULL COMMENT '触发时间小时数',
  `trigger_minute` tinyint(4) NOT NULL DEFAULT '0' COMMENT '触发时间分钟数',
  `job_id` int(11) COMMENT 'xxl-job任务ID',
  `notify_type` tinyint(4) NOT NULL COMMENT '通知方式(0-邮件)',
  `notify_title` varchar(150) NOT NULL COMMENT '邮件标题',
  `notify_content` text COMMENT '邮件内容',
  `user_id` bigint(20) NOT NULL COMMENT '订阅人ID',
  `user_name` varchar(100) NOT NULL COMMENT '订阅人姓名',
  `create_time` datetime NOT NULL COMMENT '订阅时间',
  PRIMARY KEY (`id`),
  UNIQUE KEY `data_id` (`data_id`,`data_type`,`user_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='订阅服务';

drop table if exists `visual_subscribe_push_log`;
CREATE TABLE `visual_subscribe_push_log` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `sub_title` varchar(255) NOT NULL COMMENT '标题',
  `data_type` tinyint(4) NOT NULL COMMENT '订阅内容类型',
  `notify_type` tinyint(4) NOT NULL COMMENT '通知方式(0-邮件)',
  `data_id` int(11) NOT NULL COMMENT '订阅内容ID',
  `content` text comment '发送的内容',
  `user_id` bigint(20) NOT NULL COMMENT '订阅人ID',
  `user_name` varchar(100) NOT NULL COMMENT '订阅人姓名',
  `err_code` tinyint(4) NOT NULL DEFAULT '0' COMMENT '错误码(0-成功/1-失败)',
  `err_msg` varchar(4000) DEFAULT NULL COMMENT '错误描述',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  PRIMARY KEY (`id`),
  KEY `data_id` (`data_id`,`data_type`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='订阅发送日志';

drop table if exists `visual_subscribe_notify_conf`;
CREATE TABLE `visual_subscribe_notify_conf` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `attri_key` varchar(255) DEFAULT NULL COMMENT '配置键',
  `attri_value` varchar(255) DEFAULT NULL COMMENT '值',
  `attri_name` varchar(100) DEFAULT NULL COMMENT '键名称',
  `notify_type` tinyint(4) DEFAULT NULL COMMENT '通知方式',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- 20221105- 畅玉春 添加免登录链接管理表
drop table if exists `visual_login_free_link`;
CREATE TABLE `visual_login_free_link` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `visual_data_id` int(11) DEFAULT NULL COMMENT '大屏id',
  `visual_data_title` varchar(100) DEFAULT NULL COMMENT '大屏名称',
  `link_type` int(2) DEFAULT NULL COMMENT '链接类型：0免登录 1账号匹配',
  `user_id` bigint(20) DEFAULT NULL COMMENT '账号id',
  `valid_date` date DEFAULT NULL COMMENT '有效期',
  `purpose` longtext COMMENT '用途',
  `is_verify` int(2) DEFAULT NULL COMMENT '是否验证用户有效性：0是 1否',
  `short_link` varchar(255) DEFAULT NULL COMMENT '短链接',
  `real_url` varchar(255) DEFAULT NULL COMMENT '真实链接',
  `status` int(2) DEFAULT NULL COMMENT '状态：0下线  1上线',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  `create_by` varchar(100) DEFAULT NULL COMMENT '创建人',
  `create_user_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
  `update_time` datetime DEFAULT NULL COMMENT '修改时间',
  `update_by` varchar(100) DEFAULT NULL COMMENT '修改人',
  `update_user_id` bigint(20) DEFAULT NULL COMMENT '修改人id',
  `user_token` varchar(100) DEFAULT NULL COMMENT '三方系统token',
  `access_token` varchar(100) DEFAULT NULL COMMENT '本系统token',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=6 DEFAULT CHARSET=utf8mb4 COMMENT='免登录链接';


-- 20221026 - 王坤 - 权限组添加分类id 以及 添加权限组分类表
drop table if exists data_permission_type;
create table data_permission_type(
    id int primary key auto_increment,
    type_name varchar(50) not null comment '分类名称',
    sort_num int not null default 0 comment '排序号',
    create_time datetime not null comment '创建时间',
    create_by varchar(50) not null comment '创建人姓名',
    create_user_id bigint not null comment '创建人ID'
) comment '权限组分类';

drop table if exists `data_permission`;
CREATE TABLE `data_permission` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `perm_name` varchar(100) NOT NULL COMMENT '权限组名称',
  `type_id` int(11) NOT NULL DEFAULT '0' COMMENT '分类ID',
  `perm_tag` varchar(20) DEFAULT NULL COMMENT '权限组标签',
  `perm_json` json NOT NULL COMMENT '权限组数据json',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  `create_user_id` bigint(20) NOT NULL COMMENT '创建人',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='权限组表';

-- 创建超级权限组
insert into `data_permission_type`(`id`, `type_name`, `sort_num`, `create_time`, `create_by`, `create_user_id`) values(1, '超级权限', 0, now(), '系统导入', 0);
INSERT INTO `data_permission`(`perm_name`, `type_id`, `perm_tag`, `perm_json`, `create_time`, `create_user_id`) VALUES ('超级权限组', 1, '1', '[]', now(), 0);
update `data_permission` set `id` = 0 where create_user_id = 0;

-- 20221027 - 王坤 - 添加数据源权限组和大屏权限组
drop table if exists visual_data_permission;
CREATE TABLE `visual_data_permission` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `visual_id` int(11) NOT NULL COMMENT '大屏ID',
  `data_perm_id` int(11) NOT NULL COMMENT '权限组ID',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  `create_by` varchar(50) NOT NULL COMMENT '创建人',
  `create_user_id` bigint(20) NOT NULL COMMENT '创建人ID',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='数据大屏权限组';

drop table if exists report_datasource_permission;
CREATE TABLE `report_datasource_permission` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `datasource_id` int(11) NOT NULL COMMENT '数据源ID',
  `table_names` json NOT NULL COMMENT '数据表名JSON数组',
  `data_perm_id` int(11) NOT NULL COMMENT '权限组ID',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  `create_by` varchar(50) NOT NULL COMMENT '创建人',
  `create_user_id` bigint(20) NOT NULL COMMENT '创建人ID',
  PRIMARY KEY (`id`),
  KEY `ds_id` (`datasource_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='数据源权限组';

-- 20221029 - 王坤 - 权限组与用户关系表
drop table if exists `data_permission_user`;
CREATE TABLE `data_permission_user` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `user_id` bigint(20) NOT NULL COMMENT '用户ID',
  `data_perm_id` int(11) NOT NULL COMMENT '权限组ID',
  `user_status` tinyint(4) NOT NULL DEFAULT '0' COMMENT '是否直接设置',
  PRIMARY KEY (`id`),
  UNIQUE KEY `data_perm_id` (`data_perm_id`,`user_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='权限组与用户关系表';

drop table if exists `data_permission_dept`;
CREATE TABLE `data_permission_dept` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `data_perm_id` int(11) NOT NULL COMMENT '权限组ID',
  `dept_id` bigint(20) NOT NULL COMMENT '部门ID',
  PRIMARY KEY (`id`),
  UNIQUE KEY `dept_id` (`dept_id`,`data_perm_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='权限组和部门关联表';

drop table if exists `data_permission_role`;
CREATE TABLE `data_permission_role` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `data_perm_id` int(11) NOT NULL COMMENT '权限组ID',
  `role_id` bigint(20) NOT NULL COMMENT '角色ID',
  PRIMARY KEY (`id`),
  KEY `role_id` (`role_id`,`data_perm_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='权限组和角色关联表';

-- 20221114 - 王坤 - 数据集分类添加创建人ID
ALTER TABLE `digital_operation`.`cube_workspace` 
ADD COLUMN `create_user_id` bigint(0) NOT NULL DEFAULT 0 COMMENT '创建人ID' AFTER `create_by`;

-- 20221116 - 畅玉春 - 添加第三方用户ID
alter table `blade_user` add column `third_user_id` varchar(100) comment '第三方用户id';

-- 20221129 - 畅玉春 - 自定义人群
drop table if exists `cube_define_group`;
CREATE TABLE `cube_define_group` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '自定义人群id',
  `classify_id` int(11) NOT NULL COMMENT '人群分类id',
  `group_name` varchar(100) NOT NULL COMMENT '人群名称',
  `request_freq` int(11) NOT NULL COMMENT '计算频次(0单次/1每日/2每周/3每月)',
  `define_rule` text NOT NULL COMMENT '定义规则',
  `status` int(4) DEFAULT NULL COMMENT '状态(0未开始/1计算中/2已完成)',
  `remark` varchar(100) DEFAULT NULL COMMENT '人群描述',
  `amount` bigint(20) DEFAULT NULL COMMENT '覆盖人数',
  `time_zone` varchar(100) DEFAULT NULL COMMENT '时间分区',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  `create_by` varchar(100) DEFAULT NULL COMMENT '创建人',
  `update_time` datetime DEFAULT NULL COMMENT '修改时间',
  `create_user_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
  `job_id` int(11) DEFAULT NULL COMMENT 'jobid',
  `result_table` varchar(100) DEFAULT NULL COMMENT '结果集对应表',
  `cube_columns` text COMMENT '数据集对应的字段',
  `result_columns` text COMMENT '总结果对应的字段',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='自定义人群';

drop table if exists `cube_group_classify`;
CREATE TABLE `cube_group_classify` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '人群分类id',
  `classify_name` varchar(100) NOT NULL COMMENT '人群分类名称',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  `create_by` varchar(100) NOT NULL COMMENT '创建人姓名',
  `create_user_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
  `update_time` datetime DEFAULT NULL COMMENT '修改时间',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 COMMENT='自定义人群分类';

drop table if exists `visual_login_free_link`;
CREATE TABLE `visual_login_free_link` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `visual_data_id` int(11) DEFAULT NULL COMMENT '大屏id',
  `visual_data_title` varchar(100) DEFAULT NULL COMMENT '大屏名称',
  `link_type` int(2) DEFAULT NULL COMMENT '链接类型：0免登录 1账号匹配',
  `user_id` bigint(20) DEFAULT NULL COMMENT '账号id',
  `user_name` varchar(100) DEFAULT NULL COMMENT '账号名称',
  `valid_date` date DEFAULT NULL COMMENT '有效期',
  `purpose` longtext COMMENT '用途',
  `is_verify` int(2) DEFAULT NULL COMMENT '是否验证用户有效性：0是 1否',
  `short_link` varchar(255) DEFAULT NULL COMMENT '短链接',
  `real_url` varchar(255) DEFAULT NULL COMMENT '真实链接',
  `status` int(2) DEFAULT NULL COMMENT '状态：0下线  1上线',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  `create_by` varchar(100) DEFAULT NULL COMMENT '创建人',
  `create_user_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
  `update_time` datetime DEFAULT NULL COMMENT '修改时间',
  `update_by` varchar(100) DEFAULT NULL COMMENT '修改人',
  `update_user_id` bigint(20) DEFAULT NULL COMMENT '修改人id',
  `user_token` varchar(100) DEFAULT NULL COMMENT '三方系统token',
  `access_token` text COMMENT '本系统token',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='免登录链接';

drop table if exists `visual_subscribe_operate_log`;
CREATE TABLE `visual_subscribe_operate_log` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `sub_title` varchar(255) NOT NULL COMMENT '标题',
  `data_type` tinyint(4) NOT NULL COMMENT '订阅内容类型',
  `notify_type` tinyint(4) DEFAULT NULL COMMENT '通知方式',
  `data_id` int(11) NOT NULL COMMENT '订阅内容ID',
  `operator_id` bigint(20) NOT NULL COMMENT '订阅人ID',
  `operator_name` varchar(100) NOT NULL COMMENT '订阅人姓名',
  `user_id` bigint(20) NOT NULL COMMENT '订阅人ID',
  `user_name` varchar(100) NOT NULL COMMENT '订阅人姓名',
  `operate_type` tinyint(4) DEFAULT NULL COMMENT '操作',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 COMMENT='订阅操作日志';

drop table if exists `base_visualdev_data_permission`;
CREATE TABLE `base_visualdev_data_permission` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `visualdev_id` varchar(100) NOT NULL COMMENT '表单ID',
  `visualdev_perm_name` varchar(100) NOT NULL COMMENT '表单操作权限名称',
  `data_perm_id` int(11) DEFAULT NULL COMMENT '权限组ID',
  `perm_json` text COMMENT '操作权限JSON',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  `create_user_id` bigint(20) NOT NULL COMMENT '创建人',
  `is_default` tinyint(4) NOT NULL DEFAULT '0' COMMENT '是否默认:0否 1是',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='表单操作权限';

-- 20221205 - 畅玉春 - 自定义人群添加数据集id集合字段
alter table `cube_define_group` add column `cube_ids` json comment '数据集id集合';

-- 20221203 - 王坤 - 添加清单组件导出表
CREATE TABLE `visual_component_export` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `visual_id` varchar(100) NOT NULL COMMENT '大屏ID',
  `component_id` int(11) NOT NULL COMMENT '组件ID',
  `visual_name` varchar(255) DEFAULT NULL COMMENT '大屏名称',
  `visual_category_id` int(11) DEFAULT NULL COMMENT '大屏分类ID',
  `component_name` varchar(255) DEFAULT NULL COMMENT '组件名称',
  `operator` varchar(255) DEFAULT NULL COMMENT '操作员',
  `operator_id` bigint(20) DEFAULT NULL COMMENT '操作员ID',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  `finish_time` datetime DEFAULT NULL COMMENT '完成时间',
  `export_status` tinyint(4) NOT NULL DEFAULT '0' COMMENT '任务状态（0-进行中；1-成功；2-失败）',
  `file_name` varchar(300) DEFAULT NULL COMMENT '文件名',
  `ext` text COMMENT '扩展字段，比如存储错误信息',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB CHARSET=utf8mb4 COMMENT='清单组件数据导出任务';

-- 20221207 - 王坤 - 大屏添加是否公开状态字段
alter table visual_data add column public_status tinyint NOT NULL DEFAULT 0 comment '大屏公开状态';

-- 20221214 - 王坤 - 数据集添加是否公开状态字段
ALTER TABLE `cube_data` ADD COLUMN `public_status` tinyint(4) NOT NULL DEFAULT 0 COMMENT '公开状态';

-- 20221215 - 王坤 - 创建sql视图表
CREATE TABLE `cube_sql_view` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `sql_name` varchar(100) NOT NULL COMMENT 'SQL名称',
  `view_name` varchar(50) NOT NULL COMMENT '视图名称',
  `datasource_id` int(11) NOT NULL COMMENT '数据源ID',
  `sql_statement` longtext NOT NULL COMMENT 'SQL语句',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  `create_user_id` bigint(20) NOT NULL COMMENT '创建人ID',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB COMMENT='SQL创建的数据集视图';

-- 20221228 - 王坤 - 微应用添加权限组关联表
CREATE TABLE `micro_app_data_permission` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `app_id` int(11) NOT NULL COMMENT '应用ID',
  `data_perm_id` int(11) NOT NULL COMMENT '权限组ID',
  `create_time` datetime NOT NULL COMMENT '创建时间',
  `create_by` varchar(100) NOT NULL COMMENT '创建人',
  `create_user_id` bigint(20) NOT NULL COMMENT '创建人ID',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='微应用权限组';